function result = main_function(dist_num,lx,d1,d2,delta1,delta2,N_total,ratio,t_total)
%dist_num is the distribution number
%lx is the total size in um
%d1 is the distance of Y1 from the edge of the bacterial band from the source side
%d2 is the distance of Y1 from the edge of the bacterial band from the source side
%delta11 is the width of the Y1 band region
%delta12 is the width of the Y2 band region
%drug is the antibiotic in ug/ml
%N_total is total cell number
%ratio is Y1:Y2
%t_total is the total calculation time

drug= 2000;%concentration of CRO
dt=0.1;%loop delta time in s
dx = 40;%every split size
grid_num = lx/dx + 2;%calculation matrix

%Y1 and Y2 cells growth and death
k1=0.0001;K1=2;%maximal growth rate and eigen value of Y1
k2=0.0003;K2=50;%maximal death rate  and eigen value of Y1
k4=0.0002;K4=0.1;%maximal growth rate and eigen value of Y2
k5=0.0003;K5=500;%maximal death rate  and eigen value of Y2

Dn1=430;Dn2=1800;Dn3=80;Da=350;%Diffusion coefficients of N1,N2,N3 and A
ti=t_total;ts=ti*3600;%total loop time in hours & in seconds
lam1=0.1;%conversion rate of N2
lam2 = 0.8;%conversion rate of N3
nut_gra=100;%concentration of N1

%consumption of N1,N2,N3 and A
c1=0.0005;C1=2;%consumption rate and eigen value of N1
c2=0.001;C2=0.1;%consumption rate and eigen value of N2 and N3
c3=0.02;C3=5;%consumption rate and eigen value of A

R=3.47;%Nutrients contained in a single bacterium
Nm=6*dx^2;%Maximal number of bacteria in each grid

Nsr = zeros(1,120);%sr:survival rate of All
Nsr1 = zeros(1,120);%sr1:survival rate of y1
Nsr2 = zeros(1,120);%sr2:survival rate of y2
%Record each value at each time point
Ny1=zeros(grid_num,grid_num,120);
Ny2=zeros(grid_num,grid_num,120);
NN3=zeros(grid_num,grid_num,120);
NN1=zeros(grid_num,grid_num,120);
NN2=zeros(grid_num,grid_num,120);
NA=zeros(grid_num,grid_num,120);
%Record relative change values at each time point
%Ratio of change of each item to the previous point in time
Ny1r=zeros(grid_num,grid_num,120);
Ny2r=zeros(grid_num,grid_num,120);
NN1r=zeros(grid_num,grid_num,120);
NN2r=zeros(grid_num,grid_num,120);
NN3r=zeros(grid_num,grid_num,120);
NAr=zeros(grid_num,grid_num,120);

Ny1gs=zeros(grid_num,grid_num,120);%Growth ratio of Y1
Ny1ds=zeros(grid_num,grid_num,120);%death ratio of Y1
Ny1dfs=zeros(grid_num,grid_num,120);%Diffusion ratio of Y1
Ny1s=zeros(grid_num,grid_num,120);%Ratio of change of Y1
Ny2gs=zeros(grid_num,grid_num,120);%Growth ratio of Y2
Ny2ds=zeros(grid_num,grid_num,120);%death ratio of Y2
Ny2dfs=zeros(grid_num,grid_num,120);%Diffusion ratio of Y2
Ny2s=zeros(grid_num,grid_num,120);%Ratio of change of Y2

NN1cs=zeros(grid_num,grid_num,120);%Consumption ratio of N1
NN1dfs=zeros(grid_num,grid_num,120);%Diffusion ratio of N1
NN1s=zeros(grid_num,grid_num,120);%Ratio of change of N1
NN2y1ps=zeros(grid_num,grid_num,120);%production ratio of N2
NN2y1cs=zeros(grid_num,grid_num,120);%Consumption ratio of N2 by Y1
NN2y2cs=zeros(grid_num,grid_num,120);%Consumption ratio of N2 by Y2
NN2dfs=zeros(grid_num,grid_num,120);%Diffusion ratio of N2
NN2s=zeros(grid_num,grid_num,120);%Ratio of change of N2
NN3y1ps=zeros(grid_num,grid_num,120);%production ratio of N3 by Y1
NN3y2ps=zeros(grid_num,grid_num,120);%production ratio of N3 by Y2
NN3y1cs=zeros(grid_num,grid_num,120);%Consumption ratio of N3 by Y1
NN3y2cs=zeros(grid_num,grid_num,120);%Consumption ratio of N3 by Y2
NN3dfs=zeros(grid_num,grid_num,120);%Diffusion ratio of N3
NN3s=zeros(grid_num,grid_num,120);%Ratio of change of N3

NAy2cs=zeros(grid_num,grid_num,120);%Consumption ratio of A
NAdfs=zeros(grid_num,grid_num,120);%Diffusion ratio of A
NAs=zeros(grid_num,grid_num,120);%Ratio of change of A

if Dn1*dt/(dx)^2>0.25
    warning(['Dn*dt/dx^2>0.25,',' dt=',num2str(dt),' dx=',...
        num2str(dx)]);
end

%initialization
N1=zeros(grid_num);
N2=zeros(grid_num);
N3=zeros(grid_num);
A=zeros(grid_num);
cornerN1=zeros(3,3);
cornerN2=zeros(3,3);
cornerN3=zeros(3,3);
cornerA=zeros(3,3);
[y1,y2] = ini_distr(grid_num/2,d1,d2,delta1,delta2,n,ratio);%Set the distribution of bacteria
dead_num = zeros(grid_num,grid_num);%Number and distribution of dead bacteria
for t=dt:dt:ts
    %Calculate the diffusion of corner and center
    cornerN1=diffusion_corner(cornerN1,N1,nut_gra,Dn1,dx,dt);
    N1=corner_fun(N1,cornerN1(3,3));
    cornerA=diffusion_corner(cornerA,A,drug,Da,dx,dt);
    A=corner_fun(A,cornerA(3,3));
    cornerN2=diffusion_corner(cornerN2,N2,0,Dn2,dx,dt);
    N2=corner_fun(N2,cornerN2(3,3));
    cornerN3=diffusion_corner(cornerN3,N3,0,Dn3,dx,dt);
    N3=corner_fun(N3,cornerN3(3,3));
    y1=corner_fun(y1,y1(3,3));
    y2=corner_fun(y2,y2(3,3));
    %Calculate the diffusion of edges
    N1=boundary_fun(N1);N2=boundary_fun(N2);N3=boundary_fun(N3);
    A=boundary_fun(A);
    y1=boundary_fun(y1);y2=boundary_fun(y2);
    
    %Main equations of N1 N2 A y1 y2 & N3
    N1tdfs=Dn1.*diffusion_fun(dx,N1);
    N2tdfs=Dn2.*diffusion_fun(dx,N2);
    N3tdfs=Dn3.*diffusion_fun(dx,N3);
    Atdfs=Da.*diffusion_fun(dx,A);
    N1tcs=(c1.*N1./(C1+N1+N2+N3)).*y1.*(Nm./(y1+y2+Nm));
    N1=N1+dt*(N1tdfs-N1tcs);
    
    N2ty1ps=lam1*c1*N1./(C1+N1+N2+N3).*y1.*(Nm./(y1+y2+Nm));
    N2ty1cs=(c1*N2./(C1+N1+N2+N3)).*y1.*(Nm./(y1+y2+Nm));
    N2ty2cs=(c2*N2./(C2+N2+N3)).*y2.*(Nm./(y1+y2+Nm));
    N2=N2+dt*(N2tdfs+N2ty1ps-N2ty1cs-N2ty2cs);
    
    Aty2cs=c3*A./(C3+A).*y2;
    A=A+dt*(Atdfs-Aty2cs);
    
    My1gs=(k1*(N1*(1-lam1)+N2+N3)./(K1+N1+N2+N3)).*(Nm./(y1+y2+Nm)).*y1;
    My1ds=(k2*A./(K2*((N1+N2+N3+100)./(N1+N2+N3+1))+A)).*(Nm./(y1+y2+Nm)).*y1;
    My1=My1gs-My1ds;
    My2gs=(k4*(N2+N3)./(K4+N2+N3)).*(Nm./(y1+y2+Nm)).*y2;
    My2ds=(k5*A./(K5*((N2+N3+1)./(N2+N3+0.01))+A)).*(Nm./(y1+y2+Nm)).*y2;
    My2=My2gs-My2ds;
    
    dead_num = dead_num + My1ds + My2ds;
    N3typs = dead_num.*0.0002.*R.*lam2;
    dead_num = 0.9998.*dead_num;
    N3ty1cs=(c1*N3./(C1+N1+N2+N3)).*y1.*(Nm./(y1+y2+Nm));
    N3ty2cs=(c2*N3./(C2+N2+N3)).*y2.*(Nm./(y1+y2+Nm));
    N3=N3+dt*(N3typs+N3tdfs-N3ty1cs-N3ty2cs);
    
    y1tdfs=0;
    y2tdfs=0;
    [y1,y2]=push_fun(My1,My2,y1,y2,dt);
    
    %Select a point in time to save the results
    time = 600.*(1:1:120);
    tl = find(abs(time-t)<1e-3);
    if tl>0
        if tl == 1
            Ny1r(:,:,tl)=y1./y1;
            Ny2r(:,:,tl)=y2./y2;
            NN3r(:,:,tl)=N3;
            NN1r(:,:,tl)=N1;
            NN2r(:,:,tl)=N2;
            NAr(:,:,tl)=A;
        else
            Ny1r(:,:,tl)=Ny1(:,:,tl)./Ny1(:,:,tl-1);
            Ny2r(:,:,tl)=Ny2(:,:,tl)./Ny2(:,:,tl-1);
        end
        %Record the value of each item
        Ny1(:,:,tl)=y1;
        Ny2(:,:,tl)=y2;
        NN3(:,:,tl)=N3;
        NN1(:,:,tl)=N1;
        NN2(:,:,tl)=N2;
        NA(:,:,tl)=A ;
        %Record the growth, death, diffusion and total number of Y1 and Y2
        Ny1gs(:,:,tl) = My1gs.*dt;
        Ny1ds(:,:,tl) = My1ds.*dt;
        Ny1dfs(:,:,tl) = y1tdfs.*dt;
        Ny1s(:,:,tl) = (y1tdfs+My1gs-My1ds).*dt;
        Ny2gs(:,:,tl) = My2gs.*dt;
        Ny2ds(:,:,tl) = My2ds.*dt;
        Ny2dfs(:,:,tl) = y2tdfs.*dt;
        Ny2s(:,:,tl) = (y2tdfs+My2gs-My2ds).*dt;
        %Record the consumption, diffusion, production and total value of N1,N2,N3 and A,
        %multiply by dt to get the total value of the time period
        NN1cs(:,:,tl)=N1tcs.*dt;
        NN1dfs(:,:,tl)=N1tdfs.*dt;
        NN1s(:,:,tl)=(N1tdfs-N1tcs).*dt;
        NN2y1ps(:,:,tl)=N2ty1ps.*dt;
        NN2y1cs(:,:,tl)=N2ty1cs.*dt;
        NN2y2cs(:,:,tl)=N2ty2cs.*dt;
        NN2dfs(:,:,tl)=N2tdfs.*dt;
        NN2s(:,:,tl)=(N2tdfs+N2ty1ps-N2ty1cs-N2ty2cs).*dt;
        NN3yps(:,:,tl)=N3typs.*dt;
        NN3y1cs(:,:,tl)=N3ty1cs.*dt;
        NN3y2cs(:,:,tl)=N3ty2cs.*dt;
        NN3dfs(:,:,tl)=N3tdfs.*dt;
        NN3s(:,:,tl)=(N3tdfs+N3typs-N3ty1cs-N3ty2cs).*dt;
        NAy2cs(:,:,tl) = Aty2cs.*dt;
        NAdfs(:,:,tl) = Atdfs.*dt;
        NAs(:,:,tl) = (Atdfs-Aty2cs).*dt;
        
        %Calculated the survival rate
        y1_total = sum(sum(y1(2:(grid_num-1),2:(grid_num-1))))-12*y1(3,3);
        y2_total = sum(sum(y2(2:(grid_num-1),2:(grid_num-1))))-12*y2(3,3);
        Nsr(tl) = (y1_total+y2_total)/(2*N_total);
        %Calculated the survival rate of Y1
        Nsr1(tl)=y1_total/(2*ratio*n/(ratio+1));
        %Calculated the survival rate of Y2
        Nsr2(tl)=y2_total/(2*n/(ratio+1));
    end
    A(A<0) = 0;
    N1(N1<0) = 0;
    N2(N2<0) = 0;
    N3(N3<0) = 0;
    y1(y1<0) = 0;
    y2(y2<0) = 0;
end
result = [Nsr(120),Nsr1(120),Nsr2(120)];
clear My1 My1ds My1gs My2 My2ds My2gs N1 N1tcs N1tdfs N2 N2tdfs N2ty1cs N2ty1ps N2ty2cs N3 N3tdfs...
    N3ty1cs N3ty1ps N3ty2cs N3ty2ps N3y1 N3y2 cornerA cornerN1 cornerN2 cornerN3 ts y1 y1t...
    y2 y2t N_total y1_total y2_total A Atdfs Aty2cs y1tdfs y2tdfs
outputname = ['181105_6;dsitribution=',num2str(dist_num),';N=',num2str(n)];
outputdir = fullfile(pwd,outputname);
mkdir(outputdir);
cd (outputdir)
save data
cd ..

end

